;usbtrans.s
;functions to setup and use a usb pen drive
;via the UHCI


;packet identification
%define PID_IN    0x69
%define PID_OUT   0xe1
%define PID_SETUP 0x2d

;data toggle
%define DATA0     0
%define DATA1     1


;my blue pen drive is "SimpleTech"
;I have hard coded the endpoint numbers
;these come from reading the endpoint descriptors
%define EPIN      1
%define EPOUT     2


;this is the address of our configured device
%define ADDRESS 2


align 0x10

;most command transactions use these td's
tdCommand:
times 8 dd 0  

tdData:
times 8 dd 0  

tdStatus:
times 8 dd 0  

;the device will fill in the 0x53425355 signature
;and it will copy the CBWtag
;and it will give you status: pass/fail/phase error
scsiCSW:
times 13 db 0


;the device returns information here
align 0x10
devicedata:
times 600 db 0  ;we can hold up to 1 sector + a little more

qtydevicedata dd 0


data_in  dd 0
data_out dd 0
ep_in    dd 0
ep_out   dd 0

wTotalLength dw 0
bNumConfigurations db 0
bNumInterfaces db 0
bConfigurationValue db 0
bNumEndpoints db 0
bInterfaceClass db 0
bInterfaceSubClass db 0
bEndpointAddress1 db 0
wMaxPacketSize1 dw 0
bEndpointAddress2 db 0
wMaxPacketSize2 dw 0

writenum dd 0


;********************************************************
;Control Transfers
;after you plug in the device you should
;1) reset/enable controller
;2) reset/enable port
;3) SetAddress
;4) SetConfiguration
;this is the bare minimum
;now you can do Read10, Write10, and all the scsi commands
;**********************************************************

SetAddressRequest:
db 0       ;bmRequestType
db 5       ;bRequest
dw 2       ;wValue=new usb address 
dw 0       ;wIndex
dw 0       ;wLength=bytes data returned




SetAddress:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,0         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	
	
	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],SetAddressRequest



	;**********
	;tdData
	;**********
	
	;no Data stage with SetAddress
	mov dword [qtydevicedata],0




	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN   ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,0         ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0



	call runtransaction

	ret


;*************************************************************

DeviceDescriptorRequest:
db 0x80    ;bmRequestType
db 6       ;bRequest
dw 0x0100  ;wValue=01 for DEVICE and 00 for index
dw 0       ;wIndex
dw 18      ;wLength=bytes data returned


GetDeviceDescriptor:

	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov dword [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader

	mov dword [tdCommand+8],eax
	

	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12],DeviceDescriptorRequest



	;**********
	;tdData
	;**********
	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,17        ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdData+8],eax
	

	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata


	;qty bytes device should return (needed for our paint routine)
	mov dword [qtydevicedata],18




	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT   ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdStatus+8],eax


	;4th dword of td holds pointer to send/recv data
	mov dword [tdStatus+12],0



	call runtransaction

	;save bNumConfigurations (=1 on BluePenDrive)
	mov al,[devicedata + 17]
	mov [bNumConfigurations],al

	ret



;*************************************************************


ConfigDescriptorRequest:
db 0x80    ;bmRequestType
db 6       ;bRequest=GET_DESCRIPTOR
dw 0x0200  ;wValue=02 for CONFIGURATION and 00 for index
dw 0       ;wIndex
dw 9       ;wLength=bytes data returned


;run this once requesting 9 bytes of tdData 
;then examine the wTotalLength field (byte 2,3)
;this field gives the total qty bytes which includes the 
;config + all interface + all endpoint descriptors
;then RunGetConfigDescriptor again 
;requesting wTotalLength bytes in the tdData packet
;edx = qty bytes for device to return in tdData packet

GetConfigDescriptor:


	;qty bytes device should return 
	mov dword [qtydevicedata],edx 
	mov [ConfigDescriptorRequest+6],dx



	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov dword [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	

	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12],ConfigDescriptorRequest



	;**********
	;tdData
	;**********
	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov edx, [qtydevicedata]
	dec edx           ;nbytes-1
	mov ecx,PID_IN    ;PID
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdData+8],eax
	

	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata






	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT   ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdStatus+8],eax


	;4th dword of td holds pointer to send/recv data
	mov dword [tdStatus+12],0


	call runtransaction



	;from the 1st run
	;from the Configuration Descriptor
	mov ax,[devicedata+2]
	mov [wTotalLength],ax
	mov al,[devicedata+4]
	mov [bNumInterfaces],al
	mov al,[devicedata+5]
	mov [bConfigurationValue],al

	;from the 2nd run
	;from the Interface Descriptor
	mov al,[devicedata+13]
	mov [bNumEndpoints],al
	mov al,[devicedata+14]
	mov [bInterfaceClass],al
	mov al,[devicedata+15]
	mov [bInterfaceSubClass],al
	mov al,[devicedata+20]

	;for the Endpoint Descriptor
	mov [bEndpointAddress1],al
	mov ax,[devicedata+22]
	mov [wMaxPacketSize1],ax
	mov al,[devicedata+27]
	mov [bEndpointAddress2],al
	mov ax,[devicedata+29]
	mov [wMaxPacketSize2],ax

	ret



;*********************************************************


SetConfigurationRequest:
db 0       ;bmRequestType
db 9       ;bRequest=SET_CONFIGURATION
dw 1       ;wValue=new config value  
dw 0       ;wIndex
dw 0       ;wLength=bytes data returned




SetConfiguration:


	;the configuration value to use
	;comes from the bConfigurationValue field
	;of the Configuration Descriptor
	;we know from testing a couple pen drives
	;that bNumConfigurations=1 and bConfigurationValue=1
	;so we just set it in our static structure above
	;you need to run SetAddress & SetConfiguration
	;before issueing any SCSI commands below
	

	

	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	

	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],SetConfigurationRequest



	;**********
	;tdData
	;**********
	
	;no Data stage 
	mov dword [qtydevicedata],0
	
	;just zero out the unused td
	mov dword [tdData],0
	mov dword [tdData+4],0
	mov dword [tdData+8],0
	mov dword [tdData+12],0




	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN   ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0


	call runtransaction

	ret


;*********************************************************

;first run SetConfiguration
;This is just a check to make sure we dont have 00 configuration

GetConfigurationRequest:
db 0x80    ;bmRequestType
db 8       ;bRequest=GET_CONFIGURATION
dw 0       ;wValue=new config value  
dw 0       ;wIndex
dw 1       ;wLength=bytes data returned




GetConfiguration:

	

	;**************************************
	;tdCommand
	;send command/request/token to device
	;**************************************

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	

	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],GetConfigurationRequest



	;**************************************
	;tdData
	;exchange data between host and device
	;**************************************
	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,0         ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdData+8],eax
	

	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata

	mov dword [qtydevicedata],1 



	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT   ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0



	call runtransaction

	ret


	
;*********************************************************

;this should return 00

GetMaxLunRequest:
db 0xa1    ;bmRequestType
db 0xfe    ;bRequest=Get MAX LUN
dw 0       ;wValue
dw 0       ;wIndex=interface
dw 1       ;wLength=bytes data returned



GetMaxLun:



	;**************************************
	;tdCommand
	;send command/request/token to device
	;**************************************

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	

	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],GetMaxLunRequest



	;**************************************
	;tdData
	;exchange data between host and device
	;**************************************
	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,0         ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov dword [tdData+8],eax
	

	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata

	mov dword [qtydevicedata],1 



	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT   ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0


	call runtransaction

	ret





;*********************************************************

align 0x10

;refer to ch 9 of USB spec for details 
;bmRequestType
;bit7=0 for host->device, 1=device->host
;bit0=0 for device recepient
;     1 for interface recepient
;     2 for endpoint recepient
;wValue comes from table 9-6 in USB spec 2.0
;0 = ENDPOINT_HALT
;1 = DEVICE_REMOTE_WAKEUP
;2 = TEST_MODE
;wIndex
;for endpoints we have the following:
;bit0-3 = endpoint number (should be dynamically assigned)
;bit7 is direction: 1=IN endpoint, 0=OUT endpoint
;the blue pen drive is 0x81=IN, 0x02=OUT
;wLength = tdData qty bytes to transfer 

ClearBulkInRequest:
db 2       ;bmRequestType 
db 1       ;bRequest=CLEAR_FEATURE
dw 0       ;wValue=Feature Selector=ENDPOINT_HALT
dw 0x81    ;wIndex=endpoint num
dw 0       ;wLength 




ClearBulkIn:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	

	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],ClearBulkInRequest



	;**********
	;tdData
	;**********
	
	;no data phase	
	mov dword [qtydevicedata],0
	
	;just zero out the unused td
	mov dword [tdData],0
	mov dword [tdData+4],0
	mov dword [tdData+8],0
	mov dword [tdData+12],0




	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0


	call runtransaction

	;according to the usb 1.1 spec section 5.8.5
	;"Removal of the halt condition is achieved via 
	;software intervention through a seperate control
	;pipe. This recover will reset the data toggle bit to
	;DATA0 for the endpoint on both the host and device."
	
	mov dword [data_in], 0

	ret


;*********************************************************

align 0x10

ClearBulkOutRequest:
db 2       ;bmRequestType
db 1       ;bRequest=CLEAR_FEATURE
dw 0       ;wValue=Feature Selector=ENDPOINT_HALT
dw 2       ;wIndex=endpoint number (should be dynamically assigned)
dw 0       ;wLength




ClearBulkOut:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	

	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],ClearBulkOutRequest



	;**********
	;tdData
	;**********
	
	;no Data stage 
	mov dword [qtydevicedata],0
	
	;just zero out the unused td
	mov dword [tdData],0
	mov dword [tdData+4],0
	mov dword [tdData+8],0
	mov dword [tdData+12],0




	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,0x7ff     ;nbytes-1
	mov esi,DATA1     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0



	call runtransaction

	;assume success
	mov dword [data_out], 0
	
	ret







;*********************************************************

align 0x10

BulkOnlyMassStorageResetRequest:
db 0x21    ;bmRequestType
db 0xff    ;bRequest
dw 0       ;wValue
dw 0       ;wIndex
dw 0       ;wLength




BulkOnlyMassStorageReset:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_SETUP ;PID
	mov edx,7         ;nbytes-1
	mov esi,DATA0     ;toggle
	mov edi,0         ;endpoint
	mov ebp,2         ;address
	call tdpacketheader
	mov [tdCommand+8],eax
	

	;4th dword of td holds pointer to the request
	mov dword [tdCommand+12],BulkOnlyMassStorageResetRequest



	;**********
	;tdData
	;**********
	
	;no Data stage 
	mov dword [qtydevicedata],0
	
	;just zero out the unused td
	mov dword [tdData],0
	mov dword [tdData+4],0
	mov dword [tdData+8],0
	mov dword [tdData+12],0




	;**********
	;tdStatus
	;**********

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN  ;PID
	mov edx,0x7ff   ;nbytes-1
	mov esi,DATA1   ;toggle
	mov edi,0       ;endpoint
	mov ebp,2       ;address
	call tdpacketheader
	mov [tdStatus+8],eax


	;4th dword of td holds pointer to command data
	mov dword [tdStatus+12],0


	call runtransaction

	ret


;************************************************************

;SCSI and USB
;Above this line are all the 8 byte 
;Standard USB Request Control Transfers
;which use endpoint=0
;now we deal with the SCSI request Bulk Mass Transfers
;which use the actual device endpoints for IN/OUT
;your pen drive must report in the Interface Descriptor
;bInterfaceSubClass=06 for scsi command set
;these use a 31 byte CSW sent to the device
;and a 13 byte CSW response to the host
;note for Read10 and Write10 
;the byte order varies depending on the data field see below.

;************************************************************

align 0x10

InquiryRequest:
dd 0x43425355   ;dCBWSignature
dd 0xaabbccdd   ;dCBWTag (device will copy this into CSW)
dd 0x24         ;dCBWDataTransferLength (for tdData)
db 0x80         ;bmCBWFlags 0x80=Device2Host, 00=Host2Device
db 0            ;bCBWLun
db 6            ;bCBWCBLength (of CBWCB)

;CBWCB 
db 0x12         ;operation code for INQUIRY
db 0            ;reserved
db 0            ;reserved
db 0            ;reserved
db 0x24         ;allocation length
db 0            ;control 
times 15 db 0   ;pad out 


Inquiry:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov dword [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT    ;PID
	mov edx,30         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT     
	mov ebp,ADDRESS
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]     ;toggle data_out


	;4th dword of td holds pointer to command  data
	mov dword [tdCommand+12],InquiryRequest



	;**********
	;tdData
	;**********

	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,0x23      ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS
	call tdpacketheader
	mov dword [tdData+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata


	;qty bytes device should return (needed for our paint routine)
	mov dword [qtydevicedata],36




	;**********
	;tdStatus
	;**********


	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,12        ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdStatus+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdStatus+12], scsiCSW



	call runtransaction

	ret







;************************************************************

align 0x10


;on my pen drive the CSW status is usually 01 fail
;but if I run the transaction a 2nd time it passes

TestUnitReadyRequest:
dd 0x43425355   ;dCBWSignature
dd 0xaabbccdd   ;dCBWTag
dd 0            ;dCBWDataTransferLength
db 0            ;bmCBWFlags 0x80=Device2Host, 00=Host2Device
db 0            ;bCBWLun
db 6            ;bCBWCBLength 

;CBWCB refer to spc2r20.pdf from t10
db 0            ;operation code for TEST UNIT READY
db 0            ;reserved
db 0            ;reserved
db 0            ;reserved
db 0            ;reserved
db 0            ;control ?? whats this
times 15 db 0   ;pad out 




TestUnitReady:

	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	;send 31 byte CBW via bulk OUT endpoint
	mov ecx,PID_OUT    ;PID
	mov edx,30         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT      ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]     ;toggle data_out
	

	;4th dword of td holds pointer to command  data
	mov dword [tdCommand+12],TestUnitReadyRequest



	;**********
	;tdData
	;**********

	;no Data stage 
	mov dword [qtydevicedata],0
	
	;just zero out the unused td
	mov dword [tdData],0
	mov dword [tdData+4],0
	mov dword [tdData+8],0
	mov dword [tdData+12],0

		

	;**********
	;tdStatus
	;**********


	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdStatus+4],eax


	;3rd dword of td is packet header
	;the device fills in the CSW via the bulk IN endpoint
	mov ecx,PID_IN    ;PID
	mov edx,12        ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdStatus+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdStatus+12], scsiCSW


	call runtransaction

	ret



;***********************************************************
align 0x10


RequestSenseRequest:
dd 0x43425355   ;dCBWSignature
dd 0xaabbccdd   ;dCBWTag  (just make it up)
dd 0x12         ;dCBWDataTransferLength (during tdData)
db 0x80         ;bmCBWFlags (tdData direction 0x80=IN 00=OUT)
db 0            ;bCBWLun
db 6            ;bCBWCBLength, ReadCapacity10 is a 10 byte command
;CBWCB
db 0x03         ;operation code for RequestSense
db 0            
dd 0            
db 0            
dw 0x12            
db 0            
times 6 db 0



RequestSense:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov dword [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	;send 31 byte CBW via bulk OUT endpoint
	mov ecx,PID_OUT    ;PID
	mov edx,30         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT      ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]     ;toggle data_out


	;4th dword of td holds pointer to command  data
	mov dword [tdCommand+12],RequestSenseRequest



	;**********
	;tdData
	;**********

	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,0x11      ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdData+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata


	;qty bytes device should return (needed for our paint routine)
	mov dword [qtydevicedata],0x12




	;**********
	;tdStatus
	;**********


	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN     ;PID
	mov edx,12         ;nbytes-1
	mov esi,[data_in]  ;toggle
	mov edi,EPIN       ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdStatus+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdStatus+12], scsiCSW



	call runtransaction

	ret





;************************************************************
align 0x10


;on my blue pen drive which is labeled 250Meg
;tdData returns: 0x0007b7ff00000200
;the number of blocks is 0007b7ff
;the bytes per block is 0x00000200=512 bytes
;multiply these together and convert to decimal to get 258997760

;Command Block Wrapper for SCSI ReadCapacity10 (31 bytes)
ReadCapacityRequest:
dd 0x43425355   ;dCBWSignature
dd 0xaabbccdd   ;dCBWTag  (just make it up)
dd 8            ;dCBWDataTransferLength (during tdData)
db 0x80         ;bmCBWFlags (tdData direction 0x80=IN 00=OUT)
db 0            ;bCBWLun
db 10           ;bCBWCBLength, ReadCapacity10 is a 10 byte command
;CBWCB
db 0x25         ;operation code for ReadCapacity10
db 0            
dd 0            
db 0            
dw 0            
db 0            
times 6 db 0



ReadCapacity:


	;**********
	;tdCommand
	;**********

	;1st dword of td holds pointer to next td 
	mov eax,tdData
	or eax,100b  ;depth first
	mov dword [tdCommand],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	;unlimited retries, full speed, no ISO, no IOC
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	;send 31 byte CBW via bulk OUT endpoint
	mov ecx,PID_OUT    ;PID
	mov edx,30         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT      ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]     ;toggle data_out


	;4th dword of td holds pointer to command  data
	mov dword [tdCommand+12],ReadCapacityRequest



	;**********
	;tdData
	;**********

	
	;1st dword of td holds pointer to next td 
	mov eax,tdStatus
	or eax,100b  ;depth first
	mov dword [tdData],eax
	

	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdData+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,7         ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdData+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdData+12],devicedata


	;qty bytes device should return (needed for our paint routine)
	mov dword [qtydevicedata],8




	;**********
	;tdStatus
	;**********


	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdStatus],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdStatus+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN     ;PID
	mov edx,12         ;nbytes-1
	mov esi,[data_in]  ;toggle
	mov edi,EPIN       ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdStatus+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdStatus+12], scsiCSW



	call runtransaction

	ret







;************************************************************

runtransaction:


	;to begin the usb transaction
	mov eax,tdCommand
	mov [0x01005000+4],eax

	
	;poll try for 50*100ms = 5000ms = 5 seconds then quit
	mov ecx,50


.topofloop:

	;command
	mov eax, [tdCommand+4]
	test eax,10000000000000000000000b ;bit22 set if stall
	jz .1
	jmp .decrement
.1:	test eax,100000000000000000000000b ;bit23 set if active
	jz .2
	jmp .decrement


	;data
.2:	mov eax, [tdData+4]
	test eax,10000000000000000000000b 
	jz .3
	jmp .decrement
.3:	test eax,100000000000000000000000b 
	jz .4
	jmp .decrement


	;status
.4:	mov eax, [tdStatus+4]
	test eax,10000000000000000000000b 
	jz .5
	jmp .decrement
.5:	test eax,100000000000000000000000b 
	jz .success


.decrement:
	mov eax,100
	call [SLEEP] ;for 1/10 sec
	dec ecx
	jnz .topofloop

	stc  ;set carry flag on error
	jmp .done

.success:
	clc  ;clear carry flag on success
	
	
.done:

	;detach td from framelist
	mov dword [0x01005000+4],1

	ret




;************************************************
;tdpacketheader
;build the td packet header
;this is the 3rd dword of a UHCI td
;the return value is in eax
;assign like this: mov dword [tdStatus+8],eax

;ecx = PID: 0x69=IN, 0xe1=OUT, 0x2d=SETUP
;edx = 1 less than qty bytes to send/receive 
;      0x7ff if none, 0 if send/rec one byte
;esi = 0 for DATA0 and 1 for DATA1
;edi = endpoint # (for bluepen 1=IN and 2=OUT)
;ebp = device address, use 2 for configured else 0
;************************************************

tdpacketheader:
	;qty bytes to send/receive  (n-1)
	mov eax,edx      
	shl eax,21
	;DATA0=0, DATA1=1
	mov ebx,esi        
	shl ebx,19
	or eax,ebx
	;endpoint (for blue 1=IN, 2=OUT, 0=default pipe)
	mov ebx,edi        
	shl ebx,15
	or eax,ebx
	;we use address=2 for configured pen drive else 0
	shl ebp,8
	or eax,ebp
	;PID: IN=0x69, OUT=0xe1, SETUP=0x2d
	mov al,cl
	;2nd dword of td is returned in eax
	ret
	



;***************************************************

align 0x10

Read10Request:
;the Command Block Wrapper
;the byte order of the dCBWDataTransferLength is LSB FIRST
;here the transfer length is 0x00000200=512 bytes
db 0x55,0x53,0x42,0x43, 0,0,0,0, 0,2,0,0, 0,0,10
;the CBWCB which is the scsi command block
;here the byte order for the Logical Block Address
;and the Transfer Length is LSB LAST !!!
;here we read lba=3 which is the 4th block
;and we transfer 1 block
db 0x28,0, 0,0,0,3, 0, 0,1, 0,0,0,0,0,0,0



Read10CBW:

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdCommand],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT    ;PID
	mov edx,30         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT      ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]     ;toggle data_out


	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12], Read10Request


	call run1packet

	ret



ReadData:

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdCommand],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,63        ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in

	mov dword [qtydevicedata],64


	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12], devicedata


	call run1packet

	ret




Read10CSW:

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdCommand],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,12        ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	;mov dword [tdCommand+12], scsiCSW
	mov dword [tdCommand+12], scsiCSW


	call run1packet

	ret




;************************************************************

run1packet:


	;to begin the usb transaction
	mov eax,tdCommand
	mov [0x01005000+4],eax

	
	;poll try for 50*100ms = 5000ms = 5 seconds then quit
	mov ecx,50


.topofloop:

	;command
	mov eax, [tdCommand+4]
	test eax,10000000000000000000000b ;bit22 set if stall
	jz .1  ;no stall
	jmp .decrement
.1:	test eax,100000000000000000000000b ;bit23 set if active
	jz .success  ;no active


.decrement:
	mov eax,100
	call [SLEEP] ;for 1/10 sec
	dec ecx
	jnz .topofloop

	stc  ;set carry flag on error
	jmp .done

.success:
	clc  ;clear carry flag on success
	
	
.done:

	;detach td from framelist
	mov dword [0x01005000+4],1

	ret


;******************************************************
align 0x10

Write10Request:
;the Command Block Wrapper
;the byte order of the dCBWDataTransferLength is LSB FIRST
db 0x55,0x53,0x42,0x43, 0,0,0,0, 0,2,0,0, 0,0,10
;the CBWCB which is the scsi command block
;here the byte order for the Logical Block Address
;and the Transfer Length is LSB LAST !!!
;here we write to lba=3 which is the 4th block
db 0x2a,0, 0,0,0,3, 0, 0,1, 0,0,0,0,0,0,0




Write10CBW:

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdCommand],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT    ;PID
	mov edx,30         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT      ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]     ;toggle data_out


	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12], Write10Request


	call run1packet


	ret



WriteData:

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdCommand],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_OUT    ;PID
	mov edx,63         ;nbytes-1
	mov esi,[data_out] ;toggle
	mov edi,EPOUT      ;endpoint
	mov ebp,ADDRESS    ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_out
	call [TOGGLE]  



	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12],devicedata

	;the first 64 bytes will be set to 0
	;the next 64 bytes will be set to 1 ...
	mov dword [qtydevicedata],64 
	MAC_MEMSET devicedata,[writenum],64,b 
	inc dword [writenum]

	call run1packet

	ret



Write10CSW:

	;1st dword of td holds NULL pointer because no more td's
	mov dword [tdCommand],1


	;2nd dword of td is control and status
	mov eax,1
	shl eax,23  ;active
	mov dword [tdCommand+4],eax


	;3rd dword of td is packet header
	mov ecx,PID_IN    ;PID
	mov edx,12        ;nbytes-1
	mov esi,[data_in] ;toggle
	mov edi,EPIN      ;endpoint
	mov ebp,ADDRESS   ;address
	call tdpacketheader
	mov dword [tdCommand+8],eax
	mov eax,data_in
	call [TOGGLE]     ;toggle data_in


	;4th dword of td holds pointer to send/recv data
	mov dword [tdCommand+12], scsiCSW


	call run1packet

	ret



